#ifndef CONJ_EST_HH
#define CONJ_EST_HH

#include "Estudiant.hh"
#include <vector>


class Cjt_estudiants {

private:

  // Tipus de modul: dades
  // Descripcio del tipus: representa un conjunt ordenat per DNI d'estudiants
  // Es poden consultar i modificar els seus elements (de tipus Estudiant)
  // donat un DNI o per posicio en l'ordre
    
  vector<Estudiant> vest;
  int nest;
  static const int MAX_NEST = 20;
  /*
    Invariant de la representacio:
    - vest[0..nest-1] esta ordenat creixentment pels DNI dels estudiants
    - 0 <= nest <= vest.size() = MAX_NEST
  */

  void ordenar_cjt_estudiants();
  /* Pre: cert */
  /* Post: els elements rellevants del parametre implicit estan ordenats
     creixentment pels seus DNI */
    
  static int cerca_dicot(const vector<Estudiant>& v, int left, int right, int x);
  /* Pre: v[left..right] esta ordenat creixentment per DNI,
     v.size() = n > 0, 0 <= left <= n, -1 < right < n, left <= right + 1 */
  /* Post: si a v[left..right] hi ha un element amb DNI = x, el
     resultat s una posici que hi ocupa; si no, el resultat s la
     posici que hauria d'ocupar. */

public:

  //Constructors

  Cjt_estudiants();
  /* Pre: cert */
  /* Post: el resultat es un conjunt d'estudiants buit */

   
  //Modificadors

  void afegir_estudiant(const Estudiant& est, bool& trobat);
  /* Pre: el p.i. no esta ple */
  /* Post: Si el p.i. original cont un estudiant amb el DNI d'est
     trobat s true; si no, trobat s false i s'ha afegit l'estudiant
     es al p.i. */
    
  void modificar_estudiant(const Estudiant& est, bool& trobat);
  /* Pre: cert  */
  /* Post: Si el p.i. original cont un estudiante amb el DNI d'e,
     trobat s true i aquest estudiant ha estat substituit per e al
     p.i.; si no, trobat s false i el p.i. s igual al
     p.i. original. */
    
  void modificar_iessim(int i, const Estudiant& est);
  /* Pre: 1 <= i <= nombre d'estudiants del parametre implicit,
     l'element i-essim del conjunt en ordre creixent per DNI conte
     un estudiant amb el mateix DNI que est */
  /* Post: l'estudiant est ha substituit l'estudiant i-essim
     del parametre implicit */
  
  void esborrar_estudiant(int x, bool& trobat); 
  /* Pre: cert */
  /* Post: Si el p.i. original tenia un estudiant amb DNI=x, trobat s
     true i aquest estudiant ha quedat eliminat del p.i.; si no,
     trobat s false i el p.i. s igual al p.i. original.*/


    //Consultors
	
  int mida() const;
  /* Pre: cert */
  /* Post: el resultat es el nombre d'estudiants del parametre implicit */
    
  static int mida_maxima();
  /* Pre: cert */
  /* Post: el resultat es el nombre maxim d'estudiants que pot arribar
     a tenir el parametre implicit */
    
  bool existeix_estudiant(int dni) const;
  /* Pre: dni >= 0 */
  /* Post: el resultat indica si existeix un estudiant al parmetre implicit
     amb DNI = dni */
    
  void consultar_estudiant(int x, bool& trobat, Estudiant& e) const;
  /* Pre: cert */
  /* Post: Si existeix un estudiant al p.i. amb DNI=x, trobat s true
     i e s aquest estudiant; altramente, trobat s false. */
    
  Estudiant consultar_iessim(int i) const;
  /* Pre: 1 <= i <= nombre d'estudiants que conte el parametre implicit */
  /* Post: el resultat es l'estudiant i-essim del parametre implicit
     en ordre creixent per DNI */
    

  // Lectura i escriptura
	
  void llegir();
  /* Pre: estan preparats al canal estandard d'entrada un enter (entre 0 i la
     mida maxima permesa per a la classe), que representa el nombre d'elements
     que llegirem, i les dades de tal nombre d'estudiants diferents */
  /* Post: el parametre implicit conte el conjunt d'estudiants llegits
     del canal estandard d'entrada */
    
  void escriure() const;
  /* Pre: cert */
  /* Post: s'han escrit pel canal estandard de sortida els estudiants del
     parametre implicit en ordre ascendent per DNI */
};
#endif
